if Code.ensure_loaded?(NimbleCSV) do
  defmodule ClickHouse.Format.TSVWithNames do
    @moduledoc """
    An implementation of the ClickHouse `"TSVWithNames"`/`"TabSeparatedValuesWithNames"` format.
    """

    @behaviour ClickHouse.Format

    alias ClickHouse.Format.TSV

    @parser TSV.Parser

    ################################
    # ClickHouse.Format Callbacks
    ################################

    @impl ClickHouse.Format
    @spec decode(ClickHouse.Result.t()) :: {:ok, ClickHouse.Result.t()}
    def decode(result), do: {:ok, do_decode(result)}

    @impl ClickHouse.Format
    @spec encode(ClickHouse.Query.t()) ::
            {:ok, ClickHouse.Query.t()} | {:error, ClickHouse.FormatError.t()}
    def encode(query), do: TSV.encode(query)

    ################################
    # Private API
    ################################

    defp do_decode(result) do
      [names | data] = @parser.parse_string(result.raw, skip_headers: false)
      columns = Enum.map(names, fn name -> {name, nil} end)
      %{result | columns: columns, data: data}
    end
  end
end
